import QtQuick 1.1
import qb.components 1.0

/// Edit day screen for thermostat program. Shows blocks of the programs for selected day. Allows to add / remove program block;

Screen {
	id: editDaycreen

	screenTitle: qsTr("Edit")
	isSaveCancelDialog: true

	property ThermostatApp app;
	property bool programWasEdited: false;

	QtObject {
		id: p
		// 0 == Sunday
		property int daySelected: -1;
		property int editedBlockIdx: 0;
		property bool addingNewBlock: false;
		property int maxBlocksInDay : 6
		property string blockAction: qsTr('Add');
		property int blockAnimationDuration: 400

		function mapProgramIndex(idx) {
			switch (idx) {
			case 0: return app.thermStateAway;
			case 1: return app.thermStateSleep;
			case 2: return app.thermStateActive;
			case 3: return app.thermStateRelax;
			}
		}

		// Takes Sunday based indexes of days as input
		function isYesterday(todayIdx, compareIdx) {
			var diff = todayIdx  - compareIdx
			return  ((diff === 1) || (diff == -6));
		}

		function sundayBaseToMondayBase(dayIdx) {
			var result = dayIdx - 1;
			return (result >= 0) ? result : 6;
		}

		function mondayBaseToSundayBase(dayIdx) {
			var result = dayIdx + 1;
			return (result > 6) ? 0 : result;
		}

		// input is index from the day program
		function removeBlock(index) {
			app.thermostatProgramEdited = app.thermostatUtils.deleteBlockFromSchedule(app.thermostatProgramEdited, p.daySelected, index);
			dayProgramModel.updateAddButtons();
			//programChanged();
		}

		// Argument is index in day program where new block should be added
		function addBlock(newBlockIdx) {
			p.editedBlockIdx = newBlockIdx;
			p.addingNewBlock = true;
			var tempItem = {'startMin': 0, 'startHour': 0};
			// do not use start of previous block + 30 min as initial value because previous block not started today
			if (newBlockIdx > 1) {
				var block = getBlock(0, p.editedBlockIdx - 1);
				tempItem['startMin'] = block['startMin'] + 30;
				tempItem['startHour'] = block['startHour'];
				block = getBlock(0, p.editedBlockIdx);
				if (block === null || block === undefined) {
					tempItem['endMin'] = 24;
					tempItem['endHour'] = 00;
				} else {
					tempItem['endMin'] = block['startMin'];
					tempItem['endHour'] = block['startHour'];
				}
				if (tempItem['startMin'] > 59) {
					tempItem['startHour'] += 1;
					tempItem['startMin'] -= 60;
				}
			}

			tempItem['targetState'] = getBlock(0, newBlockIdx - 1)['targetState']
			tempItem['addAboveVisible'] = false;
			tempItem['addUnderVisible'] = false;
			tempItem['addingBlockInProgress'] = true;
			dayProgramModel.insert(Math.max(newBlockIdx - 1, 0), tempItem);
			app.thermostatProgramEdited = app.thermostatUtils.addBlockToSchedule(app.thermostatProgramEdited, p.daySelected, p.editedBlockIdx);
			var endHour = 24;
			var endMin = 0;
			var block;
			if (p.editedBlockIdx + 1 < app.thermostatProgramEdited[p.daySelected].length) {
				block = getBlock(0, p.editedBlockIdx + 1);
				endHour = block.startHour;
				endMin = block.startMin;
			}
			else {
				block = getBlock(1, 0);
			}
			var endDayOfWeek = block.endDayOfWeek;

			app.thermostatProgramEdited = app.thermostatUtils.editBlockInSchedule(app.thermostatProgramEdited,
																				  p.daySelected,
																				  p.editedBlockIdx,
																				  tempItem.startMin,
																				  tempItem.startHour,
																				  p.daySelected,
																				  endMin,
																				  endHour,
																				  endDayOfWeek,
																				  tempItem.targetState);
			p.blockAction = qsTr('Add');
			if (p.editedBlockIdx == dayProgramModel.count) {
				dayProgramModel.setProperty(p.editedBlockIdx - 1, 'addingBlockInProgress', false);
				if (p.editedBlockIdx > 1) {
					dayProgramModel.setProperty(p.editedBlockIdx - 2, 'addUnderVisible', false);
				}
				afterAnimationDelay.start();
			}
			else {
				addAnimationDelay.start();
			}
		}

		// Input argument is index in day program of block to be edited.
		function editBlock(blockIdx) {
			p.editedBlockIdx = blockIdx;
			stage.openFullscreen(app.editBlockScreenUrl, {day: p.daySelected, blockIdx: blockIdx, blockAction: p.blockAction});
		}

		function getBlock(dayDelta, blockIdx) {
			if ((p.daySelected < 0) || (app.thermostatProgramEdited.length < (p.daySelected + dayDelta) % 7))
				return null;
			return app.thermostatProgramEdited[(p.daySelected + dayDelta) % 7][blockIdx];
		}

		function yesterdayIdx(todayIdx) {
			return (todayIdx - 1) >= 0 ? todayIdx - 1 : 6;
		}

		function addPreviousVisible() {
			if (p.daySelected < 0) return false;
			return (1 === app.thermostatProgramEdited[p.daySelected].length);
		}

		function previousDayText() {
			var result = "";
			if (p.daySelected >= 0)
				result = p.isYesterday(p.daySelected, p.getBlock(0,0)['startDayOfWeek']) ? qsTr('Yesterday') :
																						   i18n.daysFull[getBlock(0,0)['startDayOfWeek']]
			return result;
		}

		function previousStartTime() {
			var result = "";
			if (p.daySelected >= 0)
				result = p.getBlock(0,0)['startHour'] + ':' + (p.getBlock(0,0)['startMin'] < 10 ? '0' : '') + p.getBlock(0,0)['startMin']
			return result;
		}

		function previousProgramName() {
			var result = "";
			if (p.daySelected >= 0) result = app.thermStateName[getBlock(0,0)['targetState']];
			return result;
		}

		function previousTemperature() {
			var result = ""
			if (p.daySelected >= 0)
				result = i18n.number(app.thermStates[app.thermStatesMap[getBlock(0,0)['targetState']]]['temperature'], 1) + '°'
			return result;
		}

	}

	// day parameter is Sunday based index of day
	signal daySelected(int day)

	function blockEditCancel() {
		if (p.addingNewBlock) {
			app.thermostatProgramEdited = app.thermostatUtils.deleteBlockFromSchedule(app.thermostatProgramEdited, p.daySelected, p.editedBlockIdx);
			//dayProgramModel.remove(p.editedBlockIdx - 1);
			programChanged();
		};
		p.addingNewBlock = false;

	}

	function programChanged() {
		daySelected(p.daySelected);
	}

	function blockEdited(startHour, startMin, endHour, endMin, targetState) {
		programWasEdited = true;
		var previousBlock = p.getBlock(0, p.editedBlockIdx - 1);
		app.thermostatProgramEdited = app.thermostatUtils.editBlockInSchedule(app.thermostatProgramEdited,
																			  p.daySelected,
																			  p.editedBlockIdx,
																			  startMin,
																			  startHour,
																			  p.daySelected,
																			  endMin,
																			  endHour,
																			  p.getBlock(0, p.editedBlockIdx).endDayOfWeek,
																			  targetState);
		app.thermostatProgramEdited = app.thermostatUtils.editBlockInSchedule(app.thermostatProgramEdited,
																			  p.daySelected,
																			  p.editedBlockIdx - 1,
																			  previousBlock.startMin,
																			  previousBlock.startHour,
																			  previousBlock.startDayOfWeek,
																			  startMin,
																			  startHour,
																			  p.daySelected,
																			  previousBlock.targetState);
		//app.programOutput(app.thermostatProgramEdited, p.daySelected);
		// check if some shift of previous program needs to be done
		var blockToShift = p.editedBlockIdx;
		var shifting = blockToShift > 1;
		var newEndHour = 0;
		var newEndMin = 0;
		var newStartHour = 0;
		var newStartMin = 0;
		while (shifting) {
			shifting = false;
			blockToShift -= 1;
			if (p.getBlock(0, blockToShift).startHour * 60 + (p.getBlock(0,blockToShift).startMin) + 30 >
					p.getBlock(0, blockToShift + 1).startHour * 60 + (p.getBlock(0, blockToShift + 1).startMin)) {
				newEndHour = p.getBlock(0, blockToShift + 1).startHour;
				newEndMin = p.getBlock(0, blockToShift + 1).startMin;
				newStartHour = p.getBlock(0, blockToShift).startHour;
				newStartMin = p.getBlock(0, blockToShift).startMin;
				if ( (newEndHour * 60 + newEndMin) - (newStartHour * 60 + newStartMin) < 30) {
					var newStartTime = (newEndHour * 60 + newEndMin) - 30;
					newStartHour = Math.floor(newStartTime / 60);
					newStartMin = newStartTime % 60;
					shifting = blockToShift > 1;
				}
				app.thermostatProgramEdited = app.thermostatUtils.editBlockInSchedule(app.thermostatProgramEdited,
																					  p.daySelected,
																					  blockToShift,
																					  newStartMin,
																					  newStartHour,
																					  p.daySelected,
																					  newEndMin,
																					  newEndHour,
																					  p.daySelected,
																					  p.getBlock(0, blockToShift).targetState);


			}
		}
		// check if some shift of next program needs to be done
		blockToShift = p.editedBlockIdx;
		shifting = blockToShift < app.thermostatProgramEdited[p.daySelected].length - 1;
		while (shifting) {
			shifting = false;
			blockToShift += 1;
			if (p.getBlock(0, blockToShift).startHour * 60 + (p.getBlock(0,blockToShift).startMin) <
					p.getBlock(0, blockToShift - 1).endHour * 60 + (p.getBlock(0, blockToShift - 1).endMin)) {
				newStartHour = p.getBlock(0, blockToShift - 1).endHour;
				newStartMin = p.getBlock(0, blockToShift - 1).endMin;
				newEndHour = p.getBlock(0, blockToShift).endHour;
				newEndMin = p.getBlock(0, blockToShift).endMin;
				if ( (newEndHour * 60 + newEndMin) - (newStartHour * 60 + newStartMin) < 30) {
					var newEndTime = (newStartHour * 60 + newStartMin) + 30;
					newEndHour = Math.floor(newEndTime / 60);
					newEndMin = newEndTime % 60;
					shifting = blockToShift < app.thermostatProgramEdited[p.daySelected].length - 1;
				}
				app.thermostatProgramEdited = app.thermostatUtils.editBlockInSchedule(app.thermostatProgramEdited,
																					  p.daySelected,
																					  blockToShift,
																					  newStartMin,
																					  newStartHour,
																					  p.daySelected,
																					  newEndMin,
																					  newEndHour,
																					  p.daySelected,
																					  p.getBlock(0, blockToShift).targetState);


			}
		}
		programChanged();
		p.addingNewBlock = false;
	}

	onDaySelected: {
		if (p.daySelected >= 0)
			repeatDay.itemAt(p.sundayBaseToMondayBase(p.daySelected)).isDaySelected = false;
		p.daySelected = day;
		repeatDay.itemAt(p.sundayBaseToMondayBase(p.daySelected)).isDaySelected = true;

		dayProgramModel.clear()
		for (var program = 1; program < app.thermostatProgramEdited[p.daySelected].length; program++) {
			var tmpItem = app.thermostatProgramEdited[p.daySelected][program];
			tmpItem['addAboveVisible'] = false;
			tmpItem['addUnderVisible'] = false;
			tmpItem['addingBlockInProgress'] = false;
			dayProgramModel.append(tmpItem);
		}
		dayProgramModel.updateAddButtons();
	}

	onShown: {
		screenStateController.screenColorDimmedIsReachable = false;
	}

	onHidden: {
		addAnimationDelay.stop();
		afterAnimationDelay.stop();
	}

	onSaved: {
		// do this before the popup is shown because this state will be active again after popup
		screenStateController.screenColorDimmedIsReachable = true;
		if (programWasEdited) {
			app.programScreen.saveProgram(1);
		}
		programWasEdited = false;
	}

	onCanceled: {
		screenStateController.screenColorDimmedIsReachable = true;
		app.thermostatProgramEdited = app.thermostatProgram;
	}

	Component {
		id: dayButton
		Item {
			id: dayButtonItem
			property bool isDaySelected: false
			width: 88
			height: 36
			property string kpiPostfix: "day" + index

			RoundedRectangle {
				id: txtDay
				height: 36
				width: parent.width
				color: isDaySelected ? "#ffffff" : "#F0F0F0"
				shadowPixelSize: (isDaySelected) ? 0 : 1
				anchors {
					top: parent.top
				}
				Text {
					text: i18n.daysExtraShort[index + 1] // Monday based index
					color:  isDaySelected ? "#e64f0a" : "#565656"
					anchors.centerIn: parent
					font {
						family: qfont.semiBold.name
						pixelSize: qfont.navigationTitle
					}
				}
			}
			MouseArea {
				id: maDayButton
				anchors.fill: parent
				onClicked: {
					// emit signal with Sunday based index of the day
					daySelected(p.mondayBaseToSundayBase(index));
				}
			}
		}
	}

	ListModel {
		id: dayProgramModel

		function updateAddButtons() {
			if (count === 0) return;
			var prevBlockLength = dayProgramModel.get(0).startHour * 60 + dayProgramModel.get(0).startMin;
			var aboveVisible = prevBlockLength >= 60 && (count < p.maxBlocksInDay);
			dayProgramModel.setProperty(0, 'addAboveVisible', aboveVisible);
			dayProgramModel.setProperty(0, 'addUnderVisible', false);
			for (var i = 1; i < count; i++)
			{
				if (count >= p.maxBlocksInDay)
					dayProgramModel.setProperty(i, 'addAboveVisible', false);
				else {
					prevBlockLength = dayProgramModel.get(i).startHour * 60 + dayProgramModel.get(i).startMin;
					prevBlockLength -= dayProgramModel.get(i - 1).startHour * 60 + dayProgramModel.get(i - 1).startMin;
					dayProgramModel.setProperty(i, 'addAboveVisible', prevBlockLength >= 60);
				}
				dayProgramModel.setProperty(0, 'addUnderVisible', false);
			}
			var lastBlockStart = (dayProgramModel.get(count - 1).startHour * 60) + dayProgramModel.get(count - 1).startMin;
			var addUnderVisible = count < p.maxBlocksInDay && lastBlockStart <= 1380;
			dayProgramModel.setProperty(count - 1, 'addUnderVisible', addUnderVisible);
		}

	}

	Component {
		id: programDelegate
		Item {
			width: 664
			height: 41
			Rectangle {
				id: readyBlock
				width: 664
				height: 41
				radius: 2
				visible: !addingBlockInProgress && index < p.maxBlocksInDay
				Rectangle {
					id: modeRect
					width: 21
					height: 21
					color: app.thermStateColor[targetState]
					anchors {
						left: parent.left
						leftMargin: 10
						top: parent.top
						topMargin: 10
					}
				}
				Text {
					id: txtFrom
					text: qsTr('From')
					anchors {
						top: parent.top
						bottom: parent.bottom
						left: parent.left
						leftMargin: 57
					}
					color: "#565656"
					font {
						family: qfont.semiBold.name
						pixelSize: qfont.titleText
					}
					verticalAlignment: Text.AlignVCenter

				}
				Text {
					id: startTime
					text: startHour + ':' + (startMin < 10 ? '0' : '') + startMin
					color: "#565656"
					verticalAlignment: Text.AlignVCenter
					anchors {
						top: parent.top
						bottom: parent.bottom
						left: addTime.right
						leftMargin: Math.round(39.5 )
					}
					font {
						family: qfont.regular.name
						pixelSize: qfont.titleText
					}
				}
				Text {
					id: programName
					text: app.thermStateName[targetState]
					color: "#565656"
					verticalAlignment: Text.AlignVCenter
					anchors {
						top: parent.top
						bottom: parent.bottom
						left: parent.left
						leftMargin: 322
					}
					font {
						family: qfont.semiBold.name
						pixelSize: qfont.titleText
					}
				}
				Text {
					id: programTemperature
					text: i18n.number(app.thermStates[app.thermStatesMap[targetState]]['temperature'], 1) + '°'
					color: "#565656"
					verticalAlignment: Text.AlignVCenter
					anchors {
						top: parent.top
						bottom: parent.bottom
						right: parent.right
						rightMargin: 158
					}
					font {
						family: qfont.regular.name
						pixelSize: qfont.titleText
					}
				}

				BarButton {
					id: binButton
					imageUp: './drawables/delete_block_shadow.png'
					imageDown: './drawables/delete_block.png'
					anchors {
						right: parent.right
						rightMargin: 13
					}
					onClicked: {
						// because index will become -1 after dayProgramModel.remove(index)
						// and this remove has to be called to get animation
						if (!(addAnimationDelay.running || afterAnimationDelay.running)) {
							var indexToDel = index;
							dayProgramModel.remove(index);
							p.removeBlock(indexToDel + 1);
							programWasEdited = true;
						}
					}
				}

				BarButton {
					id: editButton
					imageUp: './drawables/edit_block_shadow.png'
					imageDown: './drawables/edit_block.png'

					anchors {
						right: binButton.left
						rightMargin: 13
					}
					onClicked: {
						if (!(addAnimationDelay.running || afterAnimationDelay.running)) {
							p.blockAction = qsTr('Edit');
							p.editBlock(index + 1);
						}
					}
					states: [
						State {
							name: "down"
							when: maEdit.pressed;
						}

					]
				}

				MouseArea {
					id: maEdit

					property string kpiPostfix: "editEntry"

					anchors.right: editButton.right
					width: 397
					height: 41
					onClicked: editButton.clicked();
				}

				// Add button above the block line
				AddTimeBtn {
					id: addTime
					width: 56
					height: 48
					upImage : './drawables/AddTimeShadow.png'
					pressedImage : './drawables/AddTime.png'
					anchors {
						left: txtFrom.right
						leftMargin: Math.round(39.5 )
						verticalCenter: parent.top
						verticalCenterOffset: -4
					}
					//visible: p.addAboveVisible(index + 1)
					visible: addAboveVisible
					onDoAction: {
						if (!(addAnimationDelay.running || afterAnimationDelay.running)) {
							p.addBlock(index + 1);
						}
					}
				}

				// Add button under the block line
				// visible only for last block in day
				AddTimeBtn {
					id: addTimeBellow
					width: 56
					height: 48

					upImage : './drawables/AddTimeShadow.png'
					pressedImage : './drawables/AddTime.png'
					visible: addUnderVisible
					anchors {
						left: txtFrom.right
						leftMargin: Math.round(39.5 )
						verticalCenter: parent.bottom
						verticalCenterOffset: 4
					}
					onDoAction: {
						if (!(addAnimationDelay.running || afterAnimationDelay.running)) {
							p.addBlock(index + 2);
						}
					}
				}
			}
			Rectangle {
				id: blockToAdd
				visible: addingBlockInProgress
				width: 664
				height: 41

				color: "transparent"
			}
		}
	}

	Item {
		width: 664
		height: 383

		anchors.centerIn: parent
		Row {
			id: dayBtnRow
			spacing: 8
			Repeater {
				id: repeatDay
				model: 7
				delegate: dayButton
			}
		}

		/**
		 * Some strange anchor items are used in this row. It is because position of the items in the row depends
		 * on positions and sizes of elements in row where program block is shown. But there are cases when no lines
		 * with blocks are visible and needs to be faked.
		 */
		Rectangle {
			id: previousProgram
			height: 31; width: parent.width
			radius: 2
			anchors {
				top: dayBtnRow.bottom
				topMargin: 16
			}
			color: "#ffffff"
			Text {
				id: previousDay
				text: p.previousDayText()
				font {
					family: qfont.regular.name
					pixelSize: qfont.bodyText
				}
				color: "#565656"
				verticalAlignment: Text.AlignVCenter
				anchors {
					top: parent.top
					bottom: parent.bottom
					left: parent.left
					leftMargin: 8
				}
			}
			Text {
				id: previousStartTime
				text: p.previousStartTime()
				font {
					family: qfont.regular.name
					pixelSize: qfont.bodyText
				}
				color: "#565656"
				verticalAlignment: Text.AlignVCenter
				anchors {
					top: parent.top
					bottom: parent.bottom
					left: parent.left
					leftMargin: 57 + anchorText.paintedWidth + 79 + 56
				}
			}
			Text {
				id: previousProgramName
				text: p.previousProgramName()
				font {
					family: qfont.regular.name
					pixelSize: qfont.bodyText
				}
				color: "#565656"
				verticalAlignment: Text.AlignVCenter
				anchors {
					top: parent.top
					bottom: parent.bottom
					left: parent.left
					leftMargin: 322
				}
			}
			Text {
				id: previousTemperature
				text: p.previousTemperature()
				font {
					family: qfont.regular.name
					pixelSize: qfont.bodyText
				}
				color: "#565656"
				verticalAlignment: Text.AlignVCenter
				anchors {
					top: parent.top
					bottom: parent.bottom
					right: parent.right
					rightMargin: 158
				}
			}
			Text {
				id: anchorText
				visible: false
				text: qsTr('From')
				font {
					family: qfont.semiBold.name
					pixelSize: qfont.titleText
				}
			}

			// Add block button under 'previous program' line
			// Visible only if there is no other block for the day
			AddTimeBtn {
				id: addTime
				width: 56
				height: 48

				upImage : './drawables/AddTimeShadow.png'
				pressedImage : './drawables/AddTime.png'
				visible: p.addPreviousVisible()
				anchors {
					left: parent.left
					leftMargin: 57 + anchorText.paintedWidth + 39.5
					verticalCenter: parent.bottom
					verticalCenterOffset: 4
				}
				onDoAction: {
					if (!(addAnimationDelay.running || afterAnimationDelay.running)) {
						p.addBlock(1);
					}
				}
			}
		}
		Column {
			width: 664
			height: 100

			spacing: 8
			anchors.top: previousProgram.bottom
			anchors.topMargin: 8
			move: Transition {
				id: blockTransition
				NumberAnimation {
					id: blockAnimation
					properties: "y"
					easing.type: Easing.Linear
					duration: p.blockAnimationDuration
				}
			}
			Repeater {
				id: repeatProgram
				model: dayProgramModel
				delegate: programDelegate
			}
		}
	}

	Timer {
		id: addAnimationDelay
		interval: p.blockAnimationDuration
		running: false
		repeat: false
		onTriggered: {
			dayProgramModel.setProperty(p.editedBlockIdx - 1, 'addingBlockInProgress', false);
			afterAnimationDelay.start();
		}
	}

	Timer {
		id: afterAnimationDelay
		interval: 300
		running: false
		repeat: false
		onTriggered: {
			p.editBlock(p.editedBlockIdx);
		}
	}
}
