import QtQuick 1.1
import qb.components 1.0

/**
	Component showing program blocks for single day together with tab-button for the day.
 */
Item {
	id: root

	property string kpiPostfix: "Thermostatday" + index
	property bool isDaySelected: false
	property bool dayEnabled: true

	// Signal emitted when day is daySelected
	// parameter dayToSelect is Sunday based index of the day
	signal daySelected(int dayToSelect);

	property int programDisplayHeight: (isNxt ? 288 : 216)

	width: 75 
	height: (isNxt ? 288 : 216) + dayButton.height // Can't use vertical scaling because hour lines need to be aligned to pixels. 288 = 24 * 12-> 24 lines of 12 pixels

	// Use a separate property to inform the 'column' component of the height. I suspect there is a reference loop somehow, but there is no
	// logging to debug the issue.
	property int extHeight: (isNxt ? 288 : 216) + dayButton.height


	QtObject {
		id: p
		property int dayProgramHeight: 0

		function isLastInDay(blockIdx) {
			return (blockIdx === programModel.count - 1);
		}

		function isFirstInDay(blockIdx) {
			var result = (blockIdx === 0) || (blockIdx === 1 && (programModel.get(blockIdx).startHour + programModel.get(blockIdx).startMin === 0))
			return result;
		}

		function calculatePxHeight() {
			if (!programModel.count) return;
			var totalHeight = 0;
			var blockHeight = 0;
			var blockLength = 0;

			// Each segment is 10 minutes
			var segmentSize = (programDisplayHeight / 24.0) / 6

			if (programModel.count === 1) {
				programModel.get(0).pxHeight = programDisplayHeight;
				return;
			}
			for (var i = 0; i < programModel.count - 1; i++) {
				blockLength = programModel.get(i + 1).startHour * 60 + programModel.get(i + 1).startMin;
				if (i > 0)
					blockLength -= programModel.get(i).startHour * 60 + programModel.get(i).startMin;
				//1.5 pixels per 10 minutes for Toon, 2 pixels per 10 minutes for NXT
				blockHeight = Math.floor(blockLength / 10 * segmentSize);
				totalHeight += blockHeight;
				programModel.get(i).pxHeight = blockHeight;
			}
			//Last block in day
			programModel.get(programModel.count - 1).pxHeight = programDisplayHeight - totalHeight;
		}
	}

	// Populate program for one day.
	function populateDayProgram(dayProgram) {
		programModel.clear();
		for(var i = 0; i < dayProgram.length; i++) {
			var programItem = dayProgram[i];
			programItem['pxHeight'] = 0;
			programModel.append(dayProgram[i]);
		}
		p.calculatePxHeight();
	}

	Component {
		id: programRectangle
		Item {
			width: 75 
			height: pxHeight;
			RoundedRectangle {
				id: rectangle
				shadowPixelSize: 0
				radius: 6
				mouseEnabled: false
				color: app.thermStateColor[targetState]
				height: parent.height - (p.isLastInDay(index) ? 0 : 2 )
				width: parent.width
				topLeftRadiusRatio: p.isFirstInDay(index) ? 1 : 0
				topRightRadiusRatio: p.isFirstInDay(index) ? 1 : 0
				bottomRightRadiusRatio: p.isLastInDay(index) ?  1 : 0
				bottomLeftRadiusRatio: p.isLastInDay(index) ? 1 : 0
				opacity: !dayEnabled ? 0.3 : (!isDaySelected ? 0.7 : 1)
				visible: height > 0

				Text {
					id: txtMode
					text: app.thermStateName[targetState]
					visible: (paintedHeight < parent.height) && isDaySelected && dayEnabled ? true : false
					color: "white"
					font.pixelSize: qfont.bodyText
					font.family: qfont.regular.name
					anchors.centerIn: parent
				}
			}
			Item {
				id: separator
				width: parent.width
				height: p.isLastInDay(index) ? 0 : 2
				anchors.bottom: parent.bottom
			}
		}
	}

	ListModel {
		id: programModel
	}

	RoundedRectangle {
		// Overlay to visualy indicate that the termostat programming is disabled.
		color: "#A8A8A8"
		opacity: 0.3
		anchors.fill: parent
		visible: !dayEnabled
	}


	Column {
		id: column
		height: parent.extHeight
		width: parent.width

		Item {
			id: dayButton
			width: parent.width
			height: 44

			RoundedRectangle {
				id: txtDay
				height: 36
				width: parent.width
				color: (isDaySelected && dayEnabled) ? "#ffffff" : "#F0F0F0"
				shadowPixelSize: (isDaySelected || !dayEnabled) ? 0 : 1
				Text {
					text: i18n.daysExtraShort[index + 1] // Monday based index
					color: { if (!dayEnabled) "#A8A8A8";
						else (isDaySelected && dayEnabled) ? "#e64f0a" : "#565656" }
					anchors.centerIn: parent
					font {
						family: qfont.semiBold.name
						pixelSize: qfont.navigationTitle
					}
				}
			}
		}

		Repeater {
			id: repeatProgram
			model: programModel
			delegate: programRectangle
			anchors.topMargin: 8;
		}
	}

	MouseArea {
		id: maColumn
		anchors.fill: column
		enabled: dayEnabled
		onClicked: {
			daySelected(app.mondayBaseToSundayBase(index));
		}
	}
}
