import QtQuick 1.1

import qb.base 1.0
import qb.components 1.0
import BasicUIControls 1.0;

Widget {
	id: thermostatSidePanel

	property string kpiPrefix: "ThermostatSidePanel."
	property ThermostatApp app;
	property alias stateInfoText: thermostatStateInfoText.text
	property alias programOn: toggle.isSwitchedOn

	property url sourceUrl: app.sidePanelUrl

	width: 248
	height: parent !== null ? parent.height : 0

	QtObject {
		id: p

		property bool setpointInProgress: false
		property real currentTemperature : 6
		property real currentSetpoint : 6
		property string errMsg: ''
		property string flameIconState: ""
		property string vhpIconState: ""
		property int spinnerAnimDuration: 500

		// when one of the buttons (setPointSpinner) is pressed
		function onTemperatureButtonPressed() {
			if (setpointInProgress === false) {
				setPointSpinner.value = currentSetpoint;
			}
			setPointSpinner.state = 'pressed';
			setpointInProgress = true;
			updateSetpointStatus();
		}

		// when one of the buttons (setPointSpinner) is released
		function onTemperatureButtonReleased() {
			var spChanged = (currentSetpoint != setPointSpinner.value)
			currentSetpoint = setPointSpinner.value;
			setpointInProgress = false;
			setPointSpinner.state = '';
			updateSetpointStatus();
			if (spChanged) {
				app.sendSetPoint(currentSetpoint);
			} else {
				// so the status goes back to what it was
				onThermostatDatasetsChanged();
			}
		}

		function activeTemperaturePreset(id) {
			// If the wizard state is not completed, we disable the temperature preset buttons.
			// This means that we shouldn't overwrite their state here.
			if (isNormalMode) {
				tempTileAway.state = (id === tempTileAway.stateId) ? "down" : "up";
				tempTileSleep.state = (id === tempTileSleep.stateId) ? "down" : "up";
				tempTileHome.state = (id === tempTileHome.stateId) ? "down" : "up";
				tempTileComfort.state = (id === tempTileComfort.stateId) ? "down" : "up";
			}
		}

		function onTemperaturePresetChanged(id) {
			activeTemperaturePreset(id);
			app.sendTempState(id);
		}

		function onThermostatDatasetsChanged() {
//Joop			currentTemperature = app.thermInfo['currentDisplayTemp'] / 100.0;
			currentTemperature = app.thermInfo['currentTemp'] / 100.0;
			currentSetpoint = app.thermInfo['currentSetpoint'] / 100.0;
			activeTemperaturePreset(app.thermInfo['activeState']);

			if (app.thermInfo.boilerModuleConnected == 0) {
				setPointError.text = qsTr("Error");
				setPointSpinner.state = 'error';
				errMsg = qsTr("Boiler adapter not connected.");
				rectStateInfo.state = 'error';
			} else if (app.thermInfo.haveOTBoiler == "1" && app.thermInfo.otCommError == "1") {
				if (app.boilerManagementFeature) {
					setPointError.text = '';
					setPointSpinner.state = 'error';
					errMsg = qsTr("There is a problem communicating with your boiler");
				} else {
					setPointError.text = qsTr("Error");
					setPointSpinner.state = 'error';
					errMsg = qsTr("Boiler not connected.");
				}
				rectStateInfo.state = 'error';
			} else if (app.thermInfo.haveOTBoiler == "1" && app.thermInfo.errorFound != 255 && !app.boilerManagementFeature) {
				setPointError.text = qsTr("Errorcode: %1").arg(app.thermInfo.errorFound);
				setPointSpinner.state = 'error';
				errMsg = qsTr("Boiler error see manual.");
				rectStateInfo.state = 'error';
			} else if (app.boilerManagementFeature && app.boilerErrorInfo.errorActive) {
				setPointError.text = '';
				setPointSpinner.state = 'error';
				if (app.boilerErrorInfo.SCDataReady)
					errMsg = app.boilerErrorInfo.errorTitle;
				else
					errMsg = qsTr("SC_timeout_title");
				rectStateInfo.state = 'error';
			} else {
				setThermostatState();
			}
			updateSetpointStatus();
			// Override the box beneath the temperature in case we're in the installation wizard.
			if (isWizardMode) {
				rectStateInfo.state = 'wizard'
			}
		}

		function updateSetpointStatus()	{
			if (setpointInProgress != true) {
				setPointSpinner.value = currentSetpoint;
			}

			if (setPointSpinner.state !== 'error') {
				if (setPointSpinner.value === currentTemperature) {
					setPointIcon.source = "drawables/triangle_right.svg";
					setPointIconDim.source = "drawables/triangle_right_dim.svg"
				} else if (setPointSpinner.value < currentTemperature) {
					setPointIcon.source = "drawables/triangle_down.svg";
					setPointIconDim.source = "drawables/triangle_down_dim.svg"
				} else {
					setPointIcon.source = "drawables/triangle_up.svg";
					setPointIconDim.source = "drawables/triangle_up_dim.svg"
				}
			}
		}

		function isStateSameAsCurrentOrNextProgramState(state) {
			if (state === app.thermStateUndef)
				return false;
			var now = new Date();
			var nowDayOfWeek = now.getDay();
			var nowTime = (now.getHours() * 60) + now.getMinutes();
			var dayOfWeekIdx = nowDayOfWeek;
			for (var i = 0; i < app.thermostatProgram[dayOfWeekIdx].length; i++) {
				// calculate the day offset between this program's start day and dayOfWeekIdx
				// negative days means it starts before dayOfWeekIdx
				var startDaysOffset = app.thermostatProgram[dayOfWeekIdx][i].startDayOfWeek - dayOfWeekIdx;
				// if we get a positive value, it means the start days is in the previous week, so we
				// adjust
				if (startDaysOffset > 0)
					startDaysOffset -= 7;
				// calculate the relative start time based on dayOfWeekIdx's midnight
				var startTime = 0;
				if (startDaysOffset < 0)
					startTime -= 1440;
				startTime += (app.thermostatProgram[dayOfWeekIdx][i].startHour * 60) + app.thermostatProgram[dayOfWeekIdx][i].startMin;
				startTime += (startDaysOffset < 0 ? startDaysOffset + 1 : startDaysOffset) * 1440;

				// calculate how many days ahead of dayOfWeekIdxthis program ends
				var endDaysOffset = app.thermostatProgram[dayOfWeekIdx][i].endDayOfWeek - dayOfWeekIdx;
				// if we get a negative value, it means the end day is in the next week, so we
				// "add" 7 days
				if (endDaysOffset < 0)
					endDaysOffset += 7;
				// calculate the relative end time based on dayOfWeekIdx's midnight
				var endTime = 0;
				if (endDaysOffset < 0)
					endTime -= 1440;
				endTime += (app.thermostatProgram[dayOfWeekIdx][i].endHour * 60) + app.thermostatProgram[dayOfWeekIdx][i].endMin;
				endTime += endDaysOffset * 1440;

				// add an offset to adjust the times according to today's midnight
				var nowOffset = dayOfWeekIdx - nowDayOfWeek;
				if (nowOffset < 0)
					nowOffset += 7
				startTime += nowOffset * 1440;
				endTime += nowOffset * 1440;

				if (startTime <= nowTime && endTime > nowTime) {
					// here we are looking at the current program

					// when states match, we know its an override of the current program with the same state
					if (app.thermostatProgram[dayOfWeekIdx][i].targetState === state) {
						return true;
					// if we are at a program that only ends at a later day, restart loop with that day and second program
					} else if (endTime >= 1440) {
						dayOfWeekIdx = (dayOfWeekIdx + Math.floor(endTime / 1440)) % 7;
						i = 0;
					}
				} else if (startTime > nowTime) {
					// here we are at the next program (after current one)

					// when the next state is the same as the current state, we got blended
					return (app.thermostatProgram[dayOfWeekIdx][i].targetState === state);
				}
			}

			return false;
		}

		function setThermostatState() {
			var thermostatStateString = '';
			setPointSpinner.state = '';
			rectStateInfo.state = 'normal';
			toggle.visible = true;

			switch(app.thermInfo['programState']) {

			case app.progStateBaseScheme:
				// when tapping hot water, keep the same text as it was before
				if (app.thermInfo['burnerInfo'] === app.burnerDhw && thermostatStateInfoText.text.length > 0) {
					thermostatStateString = thermostatStateInfoText.text;
					break;
				}

				if (app.thermInfo['burnerInfo'] === app.burnerPreheat) {
					thermostatStateString = nextBlockPreheating();
				} else {
					thermostatStateString = nextBlock();
				}
				break;

			case app.progStateTemperatureOverride:

				if (app.thermInfo['burnerInfo'] === app.burnerPreheat) {
					thermostatStateString = nextBlockPreheating();
				} else if (currentSetpoint > currentTemperature && isStateSameAsCurrentOrNextProgramState(app.thermInfo['activeState']) === false) {
					thermostatStateString = qsTr("Temporarily to %1°").arg(i18n.number(currentSetpoint, 1));
				} else {
					thermostatStateString = nextBlock();
				}
				break;

			case app.progStateManualControl:
				thermostatStateString = qsTr("Program Off");
				break;
			case app.progStateLockedBaseScheme:
				if (app.thermInfo['currentSetpoint'] > app.thermInfo['currentDisplayTemp']) {
					thermostatStateString = qsTr("To %1°").arg(i18n.number(currentSetpoint, 1));
				} else {
					thermostatStateString = qsTr("On %1°").arg(i18n.number(currentSetpoint, 1));
				}
				break;

			case app.progStateHoliday:
				thermostatStateString = qsTr("Vacation until %1").arg(vacationEnd());
				toggle.visible = false;
				break;

			default:
				break;
			}

			updateSetpointStatus();
			thermostatStateInfoText.text = thermostatStateString;
		}

		function nextBlockPreheating() {
			var nextProgramStart = new Date(app.thermInfo['nextTime'] * 1000);
			var now = new Date();
			var diff = nextProgramStart - now; // milliseconds
			diff = Math.floor((diff/1000)/60); // minutes
			diff = Math.max(Math.round(diff / 5) * 5, 5); // 5 minutes block
			if (diff >= 60) {
				var diffHr = Math.floor(diff / 60);
				var diffMin = diff % 60;
				if(diffMin !== 0)
					return qsTr("To %1° in %2h %3min.").arg(i18n.number(app.thermInfo['nextSetpoint'] / 100, 1))
					.arg(diffHr.toString())
					.arg(diffMin.toString());
				else
					return qsTr("To %1° in %2h").arg(i18n.number(app.thermInfo['nextSetpoint'] / 100, 1))
					.arg(diffHr.toString());
			} else {
				return qsTr("To %1° in %2 min.").arg(i18n.number(app.thermInfo['nextSetpoint'] / 100, 1)).arg(diff.toString());
			}
		}

		function nextBlock() {
			return qsTr("At %1 set to %2").arg(dateOrTime(app.thermInfo['nextTime'] * 1000))
			.arg(app.thermStateName[app.thermInfo['nextState']]);
		}

		function isDiffMoreThan24Hrs(compareTime) {
			var nowPlus24hrs = new Date();
			nowPlus24hrs.setDate(nowPlus24hrs.getDate() + 1);
			return compareTime > nowPlus24hrs.getTime();
		}

		function dateOrTime(time) {
			var dateOrTimeStr = ""
			var jsTime = new Date(time);

			if (isDiffMoreThan24Hrs(time)) {
				dateOrTimeStr = i18n.dateTime(jsTime, i18n.time_no | i18n.date_yes | i18n.year_no | i18n.mon_short);
			} else {
				dateOrTimeStr = i18n.dateTime(jsTime, i18n.time_yes | i18n.secs_no | i18n.hour_str_yes);
			}
			return dateOrTimeStr;
		}

		function vacationEnd() {
			var nowPlus3Years = new Date();
			nowPlus3Years.setFullYear(nowPlus3Years.getFullYear() + 3);
			var vacationEnd = new Date(app.thermInfo['nextTime'] * 1000)
			if (nowPlus3Years < vacationEnd) {
				return qsTr("I return");
			} else {
				return dateOrTime(app.thermInfo['nextTime'] * 1000);
			}
		}

		onCurrentTemperatureChanged: {
			if (setpointInProgress === false) {
				currentTempDisplay.text = i18n.number(currentTemperature, 1) + "°";
			}
		}
	}

	function init() {
		app.thermInfoChanged.connect(p.onThermostatDatasetsChanged);
		app.boilerErrorInfoChanged.connect(p.onThermostatDatasetsChanged);
		app.boilerManagementFeatureChanged.connect(p.onThermostatDatasetsChanged);
		app.heatRecoveryInfoChanged.connect(p.onThermostatDatasetsChanged);
		setPointSpinner.buttonPressed.connect(p.onTemperatureButtonPressed);
		setPointSpinner.buttonReleased.connect(p.onTemperatureButtonReleased);
		p.updateSetpointStatus();
	}

	Item {
		id: normalSidePanel
		height: parent.height
		visible: !dimState
		anchors {
			fill: parent
			margins: 8
		}

		NumberSpinner {
			id: setPointSpinner

			width: parent.width
			height: 114
			spacing: 4
			radius: 6
			buttonWidth: 74

			fontFamily: qfont.regular.name
			fontColor: colors.spText
			fontPixelSize: qfont.tileTitle
			textBaseline: 90
			alignment: StyledValueLabel.AlignmentLeft
			leftMargin: 34

			backgroundColor:            colors.spBackgroundValue
			backgroundColorButtonsUp:   colors.spBackgroundButtonsUp
			backgroundColorButtonsDown: colors.spBackgroundButtonsDown
			backgroundColorButtonsDisabled: "#F0F0F0"
			overlayButtonWhenUp:        false
			overlayColorButtonsDown:    "#ffffff"

			disableButtonAtMaximum: true
			disableButtonAtMinimum: true

			downButtonBottomClickMargin: 2

			pressingEndTime: 3000

			rangeMin: 6.0
			rangeMax: 30.0
			increment: 0.5
			valueSuffix: "°"

			mouseIsActiveInDimState: true

			upIconSource: "drawables/tempSpinner_plus.svg"
			downIconSource: "drawables/tempSpinner_minus.svg"

			Behavior on valueField.fontPixelSize {
				SmoothedAnimation {
					duration: p.spinnerAnimDuration
					velocity: -1
				}
			}

			state: ''
			states: [
				State {
					name: 'pressed'
					PropertyChanges {target: setPointIcon; anchors.topMargin: 68}
					PropertyChanges {target: setPointSpinner; valueField.fontPixelSize: qfont.timeAndTemperatureText}
					PropertyChanges {target: currentTempDisplay; anchors.topMargin: 24}
					PropertyChanges {target: currentTempDisplay; font.pixelSize: qfont.tileTitle}
				},
				State {
					name: 'error'
					PropertyChanges {target: setPointError; visible: true}
					PropertyChanges {target: setPointSpinner; fontColor: colors.spBackgroundValue}
					PropertyChanges {target: setPointIcon; source: "drawables/icon_error.svg"}
					PropertyChanges {target: setPointIconDim; source: "drawables/icon_error_dim.svg"}
					PropertyChanges {target: setPointIcon; anchors.bottomMargin: -1 }
					AnchorChanges   {target: setPointIcon; anchors.top: undefined; anchors.bottom: setPointError.baseline }
				}
			]

			Image {
				id: setPointIcon

				anchors {
					top: parent.top
					topMargin: 81
					left: parent.left
					leftMargin: 20
				}

				source: "drawables/triangle_down.svg"

				Behavior on anchors.topMargin {
					SmoothedAnimation {
						duration: p.spinnerAnimDuration
						velocity: -1
					}
				}
			}

			Text {
				id: currentTempDisplay
				anchors {
					top: parent.top
					left: parent.left
					topMargin: 12
					leftMargin: 18
				}
				font.family: qfont.regular.name
				font.pixelSize: qfont.timeAndTemperatureText
				color: colors.spText
				text: "19,0"

				Behavior on font.pixelSize {
					SmoothedAnimation {
						duration: p.spinnerAnimDuration
						velocity: -1
					}
				}

				Behavior on anchors.topMargin {
					NumberAnimation {
						duration: p.spinnerAnimDuration
					}
				}
			}

			Text {
				id: setPointError
				anchors {
					top: parent.top
					topMargin: 74
					left: setPointIcon.right
					leftMargin: 4
				}
				font.family: qfont.regular.name
				font.pixelSize: qfont.tileTitle
				color: colors.spErrorText
				text: ""
				visible: false
			}
		}

		Item {
			id: rectStateInfo
			width: parent.width
			height: 56
			anchors.top: setPointSpinner.bottom
			anchors.topMargin: 4

			Item {
				id: normalState
				anchors.fill: parent
				visible: parent.state == 'normal'
				MouseArea {
					property string kpiPostfix: "programLine." + rectStateInfo.state
					anchors.fill: parent
					onClicked: {
						if (app && app.programScreen) {
							app.programScreen.show();
						}
					}
				}

				Text {
					id: thermostatStateInfoText
					anchors {
						left: parent.left
						leftMargin: 8
						right: toggle.left
						rightMargin: anchors.leftMargin
						verticalCenter: parent.verticalCenter
					}
					font {
						family: qfont.regular.name
						pixelSize: qfont.programText
					}
					color: colors.spText
					wrapMode: Text.WordWrap
					text: "Continue on 19,0"
				}

				OnOffToggle {
					id: toggle

					anchors.verticalCenter: parent.verticalCenter
					anchors.right: parent.right

					fontFamily: qfont.semiBold.name
					fontPixelSize: qfont.titleText
					fontColor: colors.spText
					topSpacing: 8

					isSwitchedOn: app.programEnabled
					positionIsLeft: true
					leftIsSwitchedOn: false
					mouseIsActiveInDimState: true

					onSelectedChangedByUser: app.sendProgramState(isSwitchedOn)
				}
			}
			Item {
				id: errorState
				anchors.fill: parent
				visible: parent.state == 'error'
				MouseArea {
					id: mouseAreaBM
					enabled: app.boilerManagementFeature
					property string kpiPostfix: "programLine." + rectStateInfo.state
					anchors.fill: parent
					onClicked: {
						stage.openFullscreen(Qt.resolvedUrl("../boiler/BoilerScreen.qml"), {showDefault: true});
					}
				}
				Text {
					id: errorMsg
					anchors.fill: parent
					verticalAlignment: Text.AlignVCenter
					horizontalAlignment: Text.AlignHCenter
					wrapMode: Text.WordWrap
					elide: Text.ElideRight
					maximumLineCount: 3
					text: p.errMsg
					font.family: qfont.regular.name
					font.pixelSize: qfont.programText
					color: colors.spText
				}
			}
			Item {
				id: wizardState
				anchors.fill: parent
				visible: parent.state == 'wizard'
				Text {
					id: wizardMsg
					anchors.fill: parent
					wrapMode: Text.WordWrap
					font.family: qfont.regular.name
					font.pixelSize: qfont.titleText
					color: colors.spText
					text: qsTr("Here you can set the temperature")
				}
			}

			// This state determines the visibility of the 'normalState', 'errorState' and 'wizardState' Items above.
			state: 'normal'
			states: [
				State { name: 'normal' },
				State { name: 'error' },
				State { name: 'wizard' }
			]
		}

		Item {
			anchors {
				top: rectStateInfo.bottom
				topMargin: 4
			}
			width: parent.width
			height: 120

			TemperatureRectangle {
				id: tempTileAway
				anchors.top: parent.top
				anchors.left: parent.left

				topLeftRadiusRatio: 1

				subLabelText: app.thermStateName[app.thermStateAway]
				temperature: app.thermStates.thermStateAway.temperature
				stateId: app.thermStates.thermStateAway.index

				onPressed: p.onTemperaturePresetChanged(tempTileAway.stateId)
				enabled: isNormalMode
			}

			TemperatureRectangle {
				id: tempTileHome
				anchors.top: parent.top
				anchors.left: tempTileAway.right
				anchors.leftMargin: 4

				topRightRadiusRatio: 1

				subLabelText: app.thermStateName[app.thermStateActive]
				temperature: app.thermStates.thermStateActive.temperature
				stateId: app.thermStates.thermStateActive.index

				onPressed: p.onTemperaturePresetChanged(tempTileHome.stateId)
				enabled: isNormalMode
			}

			TemperatureRectangle {
				id: tempTileSleep
				anchors.top: tempTileAway.bottom
				anchors.topMargin: 4
				anchors.left: parent.left

				bottomLeftRadiusRatio: 1

				subLabelText: app.thermStateName[app.thermStateSleep]
				temperature: app.thermStates.thermStateSleep.temperature
				stateId: app.thermStates.thermStateSleep.index

				onPressed: p.onTemperaturePresetChanged(tempTileSleep.stateId)
				enabled: isNormalMode
			}

			TemperatureRectangle {
				id: tempTileComfort
				anchors.top: tempTileHome.bottom
				anchors.topMargin: 4
				anchors.left: tempTileSleep.right
				anchors.leftMargin: 4

				bottomRightRadiusRatio: 1

				subLabelText: app.thermStateName[app.thermStateRelax]
				temperature: app.thermStates.thermStateRelax.temperature
				stateId: app.thermStates.thermStateRelax.index

				onPressed: p.onTemperaturePresetChanged(tempTileComfort.stateId)
				enabled: isNormalMode
			}
			Item {
				id: blockingBanner
				anchors {
					fill: parent
					// -1 to cover temp preset shadow if any
					rightMargin: -1
					bottomMargin: -1
				}

				visible: isNormalMode && !app.thermostatStatesSaved
				MouseArea {
					id: maPresetBlocker
					anchors.fill: parent
				}
				Rectangle {
					id: blockingBannerBckg
					anchors.fill: parent
					radius: 6
					//TODO: real color
					color: "white"
					opacity: 0.6
				}
				StandardButton {
					text: qsTr("Set")
					anchors.centerIn: parent
					onClicked: {
						stage.openFullscreen(app.temperaturePresetScreenUrl);
					}
				}
			}
		}
	}

	Item {
		id: dimSidePanel
		visible: !normalSidePanel.visible
		height: parent.height
		width: parent.width

		Text {
			id: currentTempDisplayDim
			anchors {
//waste display start
//				verticalCenter: parent.top
//				verticalCenterOffset: 114
				top: parent.top
				topMargin: app.wasteIconAnyShow ? 0 : 30
//waste display end
				left: parent.left
				leftMargin: 8
			}
			font.family: qfont.regular.name
			font.pixelSize: qfont.temperatureTextDim
			color: colors.spText
			text: currentTempDisplay.text
		}

		Image {
			id: setPointIconDim
			anchors {
				bottom: setPointTextDim.baseline
				bottomMargin: -1
				left: currentTempDisplayDim.left
			}
		}

		Text {
			id: setPointTextDim
			anchors {
				baseline: currentTempDisplayDim.baseline
				baselineOffset: 40
				left: setPointIconDim.right
				leftMargin: 8
			}
			font {
				family: qfont.semiBold.name
				pixelSize: qfont.navigationTitle
			}
			color: colors.spText
			text: rectStateInfo.state === "error" ? setPointError.text : setPointSpinner.valueField.text
		}

		Text {
			id: infoTextDim
			anchors {
				baseline: setPointTextDim.baseline
				baselineOffset: 45
				left: currentTempDisplayDim.left
				right: parent.right
			}
			font {
				family: rectStateInfo.state === "error" ? qfont.regular.name : qfont.semiBold.name
				pixelSize: qfont.navigationTitle
			}
			wrapMode: Text.WordWrap
			color: colors.spText
			elide: Text.ElideRight
			maximumLineCount: 3
			text: rectStateInfo.state === "error" ? errorMsg.text : thermostatStateInfoText.text
		}
//waste display start

		Image {
			id: twasteIconzzBig
			height: 100
			width: 100
			source: app.wasteIcon
			anchors {
				left: currentTempDisplayDim.left
				right: currentTempDisplayDim.right
				top: infoTextDim.bottom
				topMargin: 20
			}
			cache: false
			fillMode: Image.PreserveAspectFit
       		visible: dimState ? app.wasteIconShow : false		// only show if one icon
		}	

		Image {
			id: twasteIconzzSmall1
			height: 90
			width: 90
			source: app.wasteIcon
			anchors {
				left: currentTempDisplayDim.left
				top: infoTextDim.bottom
				topMargin: 20
			}
			cache: false
			fillMode: Image.PreserveAspectFit
      		 	visible: dimState ? app.wasteIcon2Show : false		// only show if two icons
		}

		Image {
			id: twasteIconzzSmall2
			height: 90
			width: 90
			source: app.wasteIcon2
			anchors {
				right: currentTempDisplayDim.right
				top: infoTextDim.bottom
				topMargin: 20
			}
			cache: false
			fillMode: Image.PreserveAspectFit
  		     	visible: dimState ? app.wasteIcon2Show : false		// only show if two icons
		}
	}
}
